@@ -27,7 +27,7 @@
 #
 #========================================================================
 #
-# Version 4.1.1, released 29 Mar 2017.
+# Version 4.1.2, released 30 Apr 2017.
 #
 # See http://backuppc.sourceforge.net.
 #
@@ -42,6 +42,8 @@ use BackupPC::Storage;
 use Data::Dumper;
 use Encode;
 
+my $debugText = "";
+
 our %ConfigMenu = (
     server => {
         text  => "CfgEdit_Title_Server",
@@ -386,7 +388,8 @@ sub action
     my $Privileged = CheckPermission($host)
                        && ($PrivAdmin || $Conf{CgiUserConfigEditEnable});
     my $userHost = 1 if ( defined($host) );
-    my $debugText;
+
+    $debugText = "";
 
     if ( !$Privileged ) {
         ErrorExit(eval("qq{$Lang->{Only_privileged_users_can_edit_config_files}}"));
@@ -407,15 +410,15 @@ sub action
             # add extra variables to make the shape the same.
             #
             #print STDERR Dumper(\%In);
-            foreach my $v ( keys(%In) ) {
-                if ( $v =~ /^v_zZ_(\Q$var\E(_zZ_.*|$))/ ) {
-                    delete($In{$v}) if ( !defined($In{"orig_zZ_$1"}) );
+            foreach my $v ( sort(keys(%In)) ) {
+                if ( $v =~ /^v_((zZ_|flds_)\Q$var\E(_zZ_.*|$))/ ) {
+                    delete($In{$v}) if ( !defined($In{"orig_$1"}) );
                 }
-                if ( $v =~ /^orig_zZ_(\Q$var\E(_zZ_.*|$))/ ) {
-                    $In{"v_zZ_$1"} = $In{$v};
+                if ( $v =~ /^orig_((zZ_|flds_)\Q$var\E(_zZ_.*|$))/ ) {
+                    $In{"v_$1"} = $In{$v};
                 }
             }
-            delete($In{"vflds.$var"});
+            #delete($In{"v_flds_$var"});
         }
 
         ($newConf, $override) = inputParse($bpc, $userHost);
@@ -429,7 +432,7 @@ sub action
 	if ( $host ne "" ) {
 	    $hostConf = $bpc->ConfigDataRead($host);
 	    $override = {};
-	    foreach my $param ( keys(%$hostConf) ) {
+	    foreach my $param ( sort(keys(%$hostConf)) ) {
 		$override->{$param} = 1;
 	    }
 	} else {
@@ -611,8 +614,9 @@ EOF
 	    return false;
 	}
 	var allVars = {};
-	var varRE  = new RegExp("^v_zZ_(" + varName + ".*)");
-	var origRE = new RegExp("^orig_zZ_(" + varName + ".*)");
+	var varRE  = new RegExp("^v_((zZ_|flds_)" + varName + ".*)");
+	var origRE = new RegExp("^orig_((zZ_|flds_)" + varName + ".*)");
+	var fldsRE = new RegExp("^flds_");
         for ( var i = 0 ; i < document.editForm.elements.length ; i++ ) {
 	    var e = document.editForm.elements[i];
 	    var re;
@@ -621,7 +625,7 @@ EOF
 		    allVars[re[1]] = 0;
 		}
 		allVars[re[1]]++;
-		//debugMsg("found v_zZ_ match with " + re[1]);
+		//debugMsg("found v_ match with " + re[1]);
 		//debugMsg("allVars[" + re[1] + "] = " + allVars[re[1]]);
 	    } else if ( (re = origRE.exec(e.name)) != null ) {
 		if ( allVars[re[1]] == null ) {
@@ -636,13 +640,20 @@ EOF
 	    if ( allVars[v] != 0 ) {
 		//debugMsg("Not the same shape because of " + v);
 		sameShape = 0;
-	    } else {
+                break;
+            }
+            if ( fldsRE.exec(v) != null ) {
+                // always rebuild a compound variable
+		sameShape = 0;
+                break;
+            }
+        }
+	if ( sameShape ) {
+            for ( v in allVars ) {
                 // copy the original variable values
 		//debugMsg("setting " + v);
-		eval("document.editForm.v_zZ_" + v + ".value = document.editForm.orig_zZ_" + v + ".value");
+		eval("document.editForm.v_" + v + ".value = document.editForm.orig_" + v + ".value");
             }
-	}
-	if ( sameShape ) {
 	    return true;
 	} else {
             // need to rebuild the form since the compound variable
@@ -680,9 +691,6 @@ EOF
 
 //-->
 </script>
-
-<span id="debug">$debugText</span>
-
 EOF
 
     $content .= <<EOF;
@@ -715,7 +723,7 @@ EOF
 	if ( $host ne "" ) {
 	    $hostConf = $bpc->ConfigDataRead($host) if ( !defined($hostConf) );
             my %hostConf2 = %$hostConf;
-	    foreach my $param ( keys(%$newConf) ) {
+	    foreach my $param ( sort(keys(%$newConf)) ) {
                 if ( $override->{$param} ) {
                     $hostConf->{$param} = $newConf->{$param}
                 } else {
@@ -752,7 +760,7 @@ EOF
             }
             ($mesg, my $hostChange) = hostsDiffMesg($hostsNew);
             $bpc->HostInfoWrite($hostsNew) if ( $hostChange );
-            foreach my $host ( keys(%$copyConf) ) {
+            foreach my $host ( sort(keys(%$copyConf)) ) {
                 #
                 # Currently host names are forced to lc when they
                 # are read from the hosts file.  Therefore we need
@@ -865,12 +873,13 @@ EOF
 $errorTxt
 $tblContent
 </table>
+<span id="debug"><pre>$debugText</pre></span>
 EOF
 
     #
     # Emit all the remaining editable config settings as hidden values
     #
-    foreach my $param ( keys(%ConfigMeta) ) {
+    foreach my $param ( sort(keys(%ConfigMeta)) ) {
         next if ( $doneParam->{$param} );
         next if ( $userHost
                       && (!defined($bpc->{Conf}{CgiUserConfigEdit}{$param})
@@ -895,7 +904,7 @@ EOF
             # Emit the new settings as orig_zZ_ parameters
             #
             $doneParam = {};
-            foreach my $param ( keys(%ConfigMeta) ) {
+            foreach my $param ( sort(keys(%ConfigMeta)) ) {
                 next if ( $doneParam->{$param} );
                 next if ( $userHost
                           && (!defined($bpc->{Conf}{CgiUserConfigEdit}{$param})
@@ -913,12 +922,20 @@ EOF
             #
             # Just switching menus: copy all the orig_zZ_ input parameters
             #
-            foreach my $var ( keys(%In) ) {
-                next if ( $var !~ /^orig_zZ_/ );
-                my $val = decode_utf8($In{$var});
-                $contentHidden .= <<EOF;
+            foreach my $var ( sort(keys(%In)) ) {
+                if ( $var =~ /^orig_zZ_/ ) {
+                    my $val = decode_utf8($In{$var});
+                    $contentHidden .= <<EOF;
 <input type="hidden" name="$var" value="${EscHTML($val)}">
 EOF
+                } elsif ( $var =~ /^orig_flds_/ ) {
+                    foreach my $v ( split(/\0/, $In{$var}) ) {
+                        my $val = decode_utf8($v);
+                        $contentHidden .= <<EOF;
+<input type="hidden" name="$var" value="${EscHTML($val)}">
+EOF
+                    }
+                }
             }
 	}
     } else {
@@ -926,7 +943,7 @@ EOF
 	# First time: emit all the original config settings
 	#
 	$doneParam = {};
-        foreach my $param ( keys(%ConfigMeta) ) {
+        foreach my $param ( sort(keys(%ConfigMeta)) ) {
             next if ( $doneParam->{$param} );
             next if ( $userHost
                           && (!defined($bpc->{Conf}{CgiUserConfigEdit}{$param})
@@ -979,7 +996,8 @@ sub fieldHiddenBuild
             @order = sort(keys(%$varValue));
         }
 
-        foreach my $fld ( @order ) {
+        for ( my $fldNum = 0 ; $fldNum < @order ; $fldNum++ ) {
+            my $fld = $order[$fldNum];
             if ( defined($type->{child}) ) {
                 $childType = $type->{child}{$fld};
             } else {
@@ -989,10 +1007,10 @@ sub fieldHiddenBuild
                 # rather than hard-coded
                 #
                 $content .= <<EOF;
-<input type="hidden" name="vflds.$varName" value="${EscHTML($fld)}">
+<input type="hidden" name="${prefix}_flds_$varName" value="${EscHTML($fld)}">
 EOF
             }
-            $content .= fieldHiddenBuild($childType, "${varName}_zZ_$fld",
+            $content .= fieldHiddenBuild($childType, "${varName}_zZ_$fldNum",
                                          $varValue->{$fld}, $prefix);
         }
     } elsif ( $type->{type} eq "shortlist" ) {
@@ -1077,7 +1095,7 @@ EOF
             push(@$varValue, undef);
             $In{addVar} = "";
         }
-        $content .= "<table border=\"1\" cellspacing=\"0\">\n";
+        $content .= "<table border=\"1\" cellspacing=\"0\" class=\"editSubTable\">\n";
         my $colspan;
 
         if ( ref($type) eq "HASH" && ref($type->{child}) eq "HASH"
@@ -1139,7 +1157,7 @@ EOF
         $content .= "</td>\n";
     } elsif ( $type->{type} eq "hash" ) {
         $content .= "<td class=\"border\">\n";
-        $content .= "<table border=\"1\" cellspacing=\"0\">\n";
+        $content .= "<table border=\"1\" cellspacing=\"0\" class=\"editSubTable\">\n";
         $varValue = {} if ( ref($varValue) ne "HASH" );
 
         if ( !$isError && !$type->{noKeyEdit}
@@ -1171,7 +1189,8 @@ EOF
             @order = sort(keys(%$varValue));
         }
 
-        foreach my $fld ( @order ) {
+        for ( my $fldNum = 0 ; $fldNum < @order ; $fldNum++ ) {
+            my $fld = $order[$fldNum];
             $content .= <<EOF;
 <tr><td class="border">$fld
 EOF
@@ -1191,11 +1210,11 @@ EOF
                 # rather than hard-coded
                 #
                 $content .= <<EOF;
-<input type="hidden" name="vflds.$varName" value="${EscHTML($fld)}">
+<input type="hidden" name="v_flds_$varName" value="${EscHTML($fld)}">
 EOF
             }
             $content .= "</td>\n";
-            $content .= fieldEditBuild($childType, "${varName}_zZ_$fld",
+            $content .= fieldEditBuild($childType, "${varName}_zZ_$fldNum",
                             $varValue->{$fld}, $errors, $level + 1, undef,
 			    $isError, $onchangeSubmit,
 			    $overrideVar, $overrideSet);
@@ -1225,7 +1244,8 @@ EOF
             @order = sort(keys(%$varValue));
         }
 
-        foreach my $fld ( @order ) {
+        for ( my $fldNum = 0 ; $fldNum < @order ; $fldNum++ ) {
+            my $fld = $order[$fldNum];
             if ( defined($type->{child}) ) {
                 $childType = $type->{child}{$fld};
             } else {
@@ -1235,10 +1255,10 @@ EOF
                 # rather than hard-coded
                 #
                 $content .= <<EOF;
-<input type="hidden" name="vflds.$varName" value="${EscHTML($fld)}">
+<input type="hidden" name="v_flds_$varName" value="${EscHTML($fld)}">
 EOF
             }
-            $content .= fieldEditBuild($childType, "${varName}_zZ_$fld",
+            $content .= fieldEditBuild($childType, "${varName}_zZ_$fldNum",
                             $varValue->{$fld}, $errors, $level + 1, undef,
 			    $isError, $onchangeSubmit,
 			    $overrideVar, $overrideSet);
@@ -1317,7 +1337,7 @@ sub errorCheck
 {
     my $errors = {};
 
-    foreach my $param ( keys(%ConfigMeta) ) {
+    foreach my $param ( sort(keys(%ConfigMeta)) ) {
         fieldErrorCheck($ConfigMeta{$param}, $param, $errors);
     }
     return $errors;
@@ -1342,15 +1362,16 @@ sub fieldErrorCheck
         } elsif ( defined($type->{child}) ) {
             @order = sort(keys(%{$type->{child}}));
         } else {
-            @order = split(/\0/, $In{"vflds.$varName"});
+            @order = split(/\0/, $In{"v_flds_$varName"});
         }
-        foreach my $fld ( @order ) {
+        for ( my $fldNum = 0 ; $fldNum < @order ; $fldNum++ ) {
+            my $fld = $order[$fldNum];
             if ( defined($type->{child}) ) {
                 $childType = $type->{child}{$fld};
             } else {
                 $childType = $type->{childType};
             }
-            $ret ||= fieldErrorCheck($childType, "${varName}_zZ_$fld", $errors);
+            $ret ||= fieldErrorCheck($childType, "${varName}_zZ_$fldNum", $errors);
         }
         return $ret;
     } else {
@@ -1417,7 +1438,7 @@ sub inputParse
     my $conf     = {};
     my $override = {};
 
-    foreach my $param ( keys(%ConfigMeta) ) {
+    foreach my $param ( sort(keys(%ConfigMeta)) ) {
         my $value;
         next if ( $userHost
                       && (!defined($bpc->{Conf}{CgiUserConfigEdit}{$param})
@@ -1454,17 +1475,18 @@ sub fieldInputParse
         } elsif ( defined($type->{child}) ) {
             @order = sort(keys(%{$type->{child}}));
         } else {
-            @order = split(/\0/, $In{"vflds.$varName"});
+            @order = split(/\0/, $In{"v_flds_$varName"});
         }
 
-        foreach my $fld ( @order ) {
+        for ( my $fldNum = 0 ; $fldNum < @order ; $fldNum++ ) {
+            my $fld = $order[$fldNum];
             my $val;
             if ( defined($type->{child}) ) {
                 $childType = $type->{child}{$fld};
             } else {
                 $childType = $type->{childType};
             }
-            $ret ||= fieldInputParse($childType, "${varName}_zZ_$fld", \$val);
+            $ret ||= fieldInputParse($childType, "${varName}_zZ_$fldNum", \$val);
             last if ( $ret );
             $$value->{$fld} = $val;
         }
@@ -1531,7 +1553,7 @@ sub configDiffMesg
         $conf = "main config";
     }
 
-    foreach my $p ( keys(%ConfigMeta) ) {
+    foreach my $p ( sort(keys(%ConfigMeta)) ) {
         if ( !exists($oldConf->{$p}) && !exists($newConf->{$p}) ) {
             next;
         } elsif ( exists($oldConf->{$p}) && !exists($newConf->{$p}) ) {
@@ -1592,13 +1614,13 @@ sub hostsDiffMesg
     my $hostsOld = $bpc->HostInfoRead();
     my($mesg, $hostChange);
 
-    foreach my $host ( keys(%$hostsOld) ) {
+    foreach my $host ( sort(keys(%$hostsOld)) ) {
         if ( !defined($hostsNew->{$host}) ) {
             $mesg .= eval("qq($Lang->{CfgEdit_Log_Host_Delete})");
             $hostChange++;
             next;
         }
-        foreach my $key ( keys(%{$hostsNew->{$host}}) ) {
+        foreach my $key ( sort(keys(%{$hostsNew->{$host}})) ) {
             next if ( $hostsNew->{$host}{$key} eq $hostsOld->{$host}{$key} );
             my $valueOld = $hostsOld->{$host}{$key};
             my $valueNew = $hostsNew->{$host}{$key};
@@ -1607,7 +1629,7 @@ sub hostsDiffMesg
         }
     }
 
-    foreach my $host ( keys(%$hostsNew) ) {
+    foreach my $host ( sort(keys(%$hostsNew)) ) {
         next if ( defined($hostsOld->{$host}) );
         my $dump = Data::Dumper->new([$hostsNew->{$host}]);
         $dump->Indent(0);
