@@ -27,7 +27,7 @@
 #
 #========================================================================
 #
-# Version 4.1.1, released 29 Mar 2017.
+# Version 4.1.2, released 30 Apr 2017.
 #
 # See http://backuppc.sourceforge.net.
 #
@@ -330,7 +330,12 @@ use vars qw(%ConfigMeta);
     ArchivePostUserCmd	=> {type => "string", undefIfEmpty => 1},
     UserCmdCheckStatus  => "boolean",
 
-    ClientNameAlias 	=> {type => "string", undefIfEmpty => 1},
+    ClientNameAlias 	=> {
+	    type    => "list",
+	    emptyOk => 1,
+	    undefIfEmpty => 1,
+	    child   => "string",
+    },
 
     ######################################################################
     # Email reminders, status and messages
